// Copyright Maciej Sobczak 2008-2020.
// This file is part of YAMI4.
//
// YAMI4 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// YAMI4 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.

#include "../network_utils.h"
#include <yami4-core/allocator.h>
#include <yami4-core/core.h>
#include "../io_error_handler.h"

#include <cstdio>
#include <cstring>

using namespace yami;
using namespace details;

core::result details::get_host_by_name(const char * hostname, host_type & host,
    core::io_error_function io_error_callback, void * io_error_callback_hint)
{
    int a, b, c, d;
    core::result res;

    int cc = sscanf(hostname, "%d.%d.%d.%d", &a, &b, &c, &d);
    if (cc == 4 &&
        a >= 0 && a <= 255 &&
        b >= 0 && b <= 255 &&
        c >= 0 && c <= 255 &&
        d >= 0 && d <= 255)
    {
        host = IP_ADDRESS(a, b, c, d);
        
        res = core::ok;
    }
    else
    {
        handle_io_error("parse IP address",
            io_error_callback, io_error_callback_hint);

        res = core::io_error;
    }

    return res;
}

port_type details::get_network_port(long port)
{
    // NetX uses port numbers in host order:
    return static_cast<port_type>(port);
}
