// Copyright Maciej Sobczak 2008-2020.
// This file is part of YAMI4.
//
// YAMI4 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// YAMI4 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.

#include "flag.h"
#include <yami4-core/fatal_errors.h>

using namespace yami;
using namespace details;

details::flag::flag()
{
    flags_ = xEventGroupCreate(); 
    if (flags_ == NULL)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

details::flag::~flag()
{
    vEventGroupDelete(flags_);
}

void details::flag::notify()
{
    (void)xEventGroupSetBits(flags_, 1ul);
}

void details::flag::suppress()
{
    (void)xEventGroupClearBits(flags_, 1ul);
}

void details::flag::wait()
{
    (void)xEventGroupWaitBits(flags_, 1ul, pdTRUE, pdFALSE, portMAX_DELAY);
}

bool details::flag::wait(std::size_t relative_timeout)
{
    bool result;
    
    EventBits_t actual_flags = xEventGroupWaitBits(flags_, 1ul, pdTRUE, pdFALSE, relative_timeout);
    if (actual_flags != 0ul)
    {
        result = true;
    }
    else
    {
        result = false;
    }
    
    return result;
}

bool details::flag::wait_absolute(unsigned long long timeout)
{
    // absolute waits are not supported for FreeRTOS
    
    fatal_failure(__FILE__, __LINE__);
    
    // dummy return
    return false;
}
