// Copyright Maciej Sobczak 2008-2020.
// This file is part of YAMI4.
//
// YAMI4 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// YAMI4 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.

#include "flag.h"
#include <yami4-core/fatal_errors.h>

using namespace yami;
using namespace details;

details::flag::flag()
{
    UINT cc = tx_event_flags_create(&flags_, const_cast<char *>("YAMI4 Flags"));
    if (cc != TX_SUCCESS)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

details::flag::~flag()
{
    UINT cc = tx_event_flags_delete(&flags_);
    if (cc != TX_SUCCESS)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

void details::flag::notify()
{
    UINT cc = tx_event_flags_set(&flags_, 1ul, TX_OR);
    if (cc != TX_SUCCESS)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

void details::flag::suppress()
{
    UINT cc = tx_event_flags_set(&flags_, 0ul, TX_AND);
    if (cc != TX_SUCCESS)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

void details::flag::wait()
{
    ULONG actual_flags;
    UINT cc = tx_event_flags_get(&flags_, 1ul, TX_OR, &actual_flags, TX_WAIT_FOREVER);
    if (cc != TX_SUCCESS)
    {
        fatal_failure(__FILE__, __LINE__);
    }
}

bool details::flag::wait(std::size_t relative_timeout)
{
    bool result;
    
    ULONG actual_flags;
    UINT cc = tx_event_flags_get(&flags_, 1ul, TX_OR, &actual_flags, relative_timeout);
    if (cc == TX_SUCCESS)
    {
        result = true;
    }
    else if (cc == TX_NO_EVENTS)
    {
        result = false;
    }
    else
    {
        fatal_failure(__FILE__, __LINE__);
    }
    
    return result;
}

bool details::flag::wait_absolute(unsigned long long timeout)
{
    // absolute waits are not supported for ThreadX
    
    fatal_failure(__FILE__, __LINE__);
    
    // dummy return
    return false;
}
