// Copyright Maciej Sobczak 2008-2020.
// This file is part of YAMI4.
//
// YAMI4 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// YAMI4 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.

#include "flag.h"
#include <yami4-core/fatal_errors.h>

#include <ti/sysbios/BIOS.h>

using namespace yami;
using namespace details;

details::flag::flag()
{
    Event_construct(&flags_, NULL);
}

details::flag::~flag()
{
    Event_destruct(&flags_);
}

void details::flag::notify()
{
    Event_post(Event_Handle(&flags_), 1ul);
}

void details::flag::suppress()
{
    // Event_pend "consumes" events, which is expected here:

    (void)Event_pend(Event_Handle(&flags_), 1ul, 0ul, 0);
}

void details::flag::wait()
{
    // flag is not supposed to be "consumed" by the wait operation,
    // but since Event_pend clears the events, the flag state needs to be restored
    //
    // note: the risk of hazard is minimized, as it can happen only
    // in the flow manager, which allows "soft" overflows, and will
    // eventually set the flag as expected even if it was accidentally restored
    // to the wrong value here

    UInt currentEvents = Event_pend(Event_Handle(&flags_), 1ul, 0ul, BIOS_WAIT_FOREVER);
    Event_post(Event_Handle(&flags_), currentEvents);
}

bool details::flag::wait(std::size_t relative_timeout)
{
    bool result;
    
    UInt consumed = Event_pend(Event_Handle(&flags_), 1ul, 0ul, relative_timeout);
    if (consumed != 0u)
    {
        result = true;
    }
    else
    {
        result = false;
    }
    
    return result;
}

bool details::flag::wait_absolute(unsigned long long timeout)
{
    // absolute waits are not supported for TI-RTOS
    
    fatal_failure(__FILE__, __LINE__);
    
    // dummy return
    return false;
}
