// Copyright Maciej Sobczak 2008-2020.
// This file is part of YAMI4.
//
// YAMI4 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// YAMI4 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.

#ifndef YAMI4BROKER_CONFIGURATION_H
#define YAMI4BROKER_CONFIGURATION_H

#include "broker.h"
#include "../common/log.h"

#include <yami4-core/parameters.h>

#include <string>

namespace broker
{

// Initializes all configuration options from the config file
// that is named in the first command-line parameter or
// from the yami4broker.cfg file if no command-line argument is given.
bool init_config(int argc, char * argv[]);

// Returns the listener endpoint for the messaging part.
std::string listener();

// Returns the file name where the resolved listener target should be written.
std::string announce_file();

// Returns the warmup time
// (time between start to when the broker accepts messages).
std::size_t warmup_time();

// Max size of subscription table.
std::size_t max_subscriptions();

// Max queue length for any given subscriber.
std::size_t max_client_queue();

// Queue overflow policy.
overflow_policy subscription_overflow_policy();

// Log file name. If not specified, log is printed on standard output.
std::string log_file_name();

// Max size, in entries of the log, after which a new file is created.
// If not specified, the log file grows indefinitely and does not have
// the timestamp attached to its name.
std::size_t log_file_max_size();

// Whether the log file should be synchronized after each line.
bool log_file_line_sync();

// Initial log level.
bool log_enabled(logger::module m);

// Push channels configuration.
std::string forward_target(std::size_t index);
std::string forward_filter(std::size_t index);

// Subscription redirection
// - if non-empty, request for subscription is redirected
// to one of the listed targets.
std::string subscription_redirection_targets(const std::string & tags);

// Agent options.
void agent_options(yami::core::parameters & options);

} // namespace broker

#endif // YAMI4BROKER_CONFIGURATION_H
